/* HSM Object Management Functions */

#ifndef _LIBPKI_HSM_MAIN_H
#define _LIBPKI_HSM_MAIN_H

HSM *HSM_new( char *dir, char *name );
HSM *HSM_new_init ( char *driver, char *name, PKI_STACK *pre_cmds,
							PKI_STACK *post_cmds );
int HSM_free ( HSM *hsm );
// int HSM_load( int type, char *url_s, HSM **hsm );

/* Signing Functions Prototypes */
int PKI_X509_CERT_sign( PKI_X509_CERT *x, PKI_KEYPAIR *key,
		PKI_DIGEST_ALG *digest, HSM *hsm );

int PKI_X509_REQ_sign ( PKI_X509_REQ *x, PKI_KEYPAIR *key,
		PKI_DIGEST_ALG *digest, HSM *hsm );

int PKI_X509_CRL_sign ( PKI_X509_CRL *x, PKI_KEYPAIR *key, 
		PKI_DIGEST_ALG *digest, HSM *hsm );

PKI_KEYPAIR *PKI_KEYPAIR_get_url ( URL *url, PKI_CRED *cred, HSM *hsm );

int PKI_sign ( PKI_OBJTYPE type, void *x, void *it,
			PKI_ALGOR *alg, PKI_STRING *bit,
			PKI_KEYPAIR *key, PKI_DIGEST_ALG *digest,
			HSM *hsm );
int PKI_verify ( PKI_OBJTYPE type, void *x, PKI_KEYPAIR *key, HSM *hsm );

/* Generate a new Keypair */
PKI_KEYPAIR *PKI_KEYPAIR_new( int type, int bits, char *label,
					PKI_CRED *cred, HSM *hsm );
int PKI_KEYPAIR_free( PKI_KEYPAIR *key, HSM *hsm );

/* Get Keypair */
PKI_KEYPAIR *PKI_KEYPAIR_get ( char *url_s, PKI_CRED *cred, HSM *hsm );
PKI_KEYPAIR *PKI_KEYPAIR_get_url ( URL *url, PKI_CRED *cred, HSM *hsm );

/* Export Keypair to a URL */
int PKI_KEYPAIR_export(PKI_KEYPAIR *x, char *label, int format, PKI_CRED *cred,
						char *url_string, HSM *hsm);
int PKI_KEYPAIR_export_url(PKI_KEYPAIR *x, char *label, int format, 
				PKI_CRED *cred, URL *url, HSM *hsm);

/* Added MACRO to ease usage of the general signature function PKI_sign() */
#define PKI_OBJ_sign( t,x,k,d,h ) PKI_sign (t,x,NULL,NULL,NULL,k,d,h)

#define PKI_ITEM_sign(t,x,i,a,b,k,d,h) \
		PKI_sign(t,x,(void *) ASN1_ITEM_rptr(i),a,b,k,d,h)

#endif
