
#ifndef _LIBPKI_HSM_ST_H
#define _LIBPKI_HSM_ST_H

typedef enum {
	HSM_TYPE_OTHER		= 0,
	HSM_TYPE_SOFTWARE,
	HSM_TYPE_ENGINE,
	HSM_TYPE_KMF,
	HSM_TYPE_PKCS11
} PKI_HSM_TYPE;

typedef enum {
	PKI_OBJTYPE_UNKNOWN 	= 0,
	PKI_OBJTYPE_X509_CERT,
	PKI_OBJTYPE_X509_REQ,
	PKI_OBJTYPE_X509_CRL,
	PKI_OBJTYPE_PKCS7,
	PKI_OBJTYPE_PKCS12,
	PKI_OBJTYPE_PKI_MSG,
	PKI_OBJTYPE_SCEP_MSG,
	PKI_OBJTYPE_CMS_MSG,
	PKI_OBJTYPE_PRQP_REQ,
	PKI_OBJTYPE_PRQP_RESP
} PKI_OBJTYPE;

// typedef hsm_st;
// typedef callbacks_st;

typedef struct callbacks_st {
	/* ------------- HSM Management functions --------------- */
	/* HSM initialization function */
	int      	(*init) (void *driver, PKI_CONFIG *);

	/* HSM free function */
	int             (*free) (void *driver);

	/* ------------- Signing functions --------------- */
	/* General Signing function */
	int		(*sign)(PKI_OBJTYPE type, void *, void *, 
				PKI_ALGOR *, PKI_STRING *,
				PKI_KEYPAIR *, PKI_DIGEST_ALG *, 
							void *driver );

	/* General Verify Function */
	int		(*verify)(PKI_OBJTYPE type, void *, PKI_KEYPAIR *,
							void * driver );

	/* ------------- Key Management functions --------------- */
	/* Key generation function */
       	PKI_KEYPAIR	* (*keypair_new)( int, int, char*, PKI_CRED *, 
								void *driver);

	/* Free Memory associated with a Key */
	int		  (*keypair_free)( PKI_KEYPAIR *, void *driver);

	/* Key remove function - supported on some HSMs */
	int		(*keypair_remove)( PKI_KEYPAIR *, char *label, 
						PKI_CRED *, void *driver );

	/* Key get function */
       	PKI_KEYPAIR	* (*keypair_get_url)( URL *, PKI_CRED *, void *driver);

	/* Key put function - export key somewhere */
       	int		(*keypair_export_url)(PKI_KEYPAIR *, char *, int format, URL *, PKI_CRED *, void *driver);
	/* ------------- Certificate Management functions --------------- */
	// To Be Defined!!!!!!!!

} HSM_CALLBACKS;

/* Structure for PKI_TOKEN definition */
typedef struct hsm_st {

	/* Version of the token */
	int version;

	/* Description of the HSM */
	char *description;

	/* Manufacturer */
	char *manufacturer;

	/* Pointer to the HSM config file and parsed structure*/
	PKI_CONFIG *config;

	/* One of PKI_HSM_TYPE value */
	PKI_HSM_TYPE type;

	/* ID of the driver - this is used to identify the driver
	   to be used, e.g., 'id://LunaCA' for loading the ENGINE
	   LunaCA extension */
	URL *id;

	/* Pointer to the internal structure for drivers */
	void *driver;

	HSM_CALLBACKS *callbacks;
} HSM;

/* End of _LIBPKI_HSM_ST_H */
#endif
