/* openca/pkicrypto/url.h */
/*
 * LIBPKI - OpenSource PKI library
 * by Massimiliano Pala (madwolf@openca.org) and OpenCA project
 *
 * Copyright (c) 2001-2007 The OpenCA Project.  All rights reserved.
 *
 * ====================================================================
 *
 * This product includes cryptographic software written by Eric Young
 * (eay@cryptsoft.com).  This product includes software written by Tim
 * Hudson (tjh@cryptsoft.com).
 *
 */


#ifndef _LIBPKI_URL_H
#define _LIBPKI_URL_H

#define URI_PROTO_FILE			0
#define URI_PROTO_LDAP			1
#define URI_PROTO_HTTP			2
#define URI_PROTO_FTP			3
#define URI_PROTO_ID			5
#define URI_PROTO_MYSQL			10
#define URI_PROTO_PG			20
#define URI_PROTO_PKCS11		30

#define DEFAULT_LDAP_PORT		389
#define DEFAULT_HTTP_PORT		80
#define DEFAULT_FTP_PORT		21
#define DEFAULT_MYSQL_PORT		3306
#define DEFAULT_PG_PORT			3456
#define DEFAULT_PKCS11_PORT		-1

#include <libpki/stack.h>

typedef struct url_data
	{
		/* Original URL string */
		char * url_s;

		/* Protocol, currently supported LDAP and FILE */
		int proto;

		/* Address or filename */
		char *addr;

		/* Communication Port (where supported by the protocol) */
		int port;

		/* Authentication (where supported by the protocol) */
		char *usr;
		char *pwd;

		/* Search facility - for LDAP the DN is in the path, while
		   the attributes for filtering the responses are here in
		   the attrs stack. The same for mysql:// or postgres://
		   urls */
		char *attrs;

		/* HTTP specific */
		char *path;
	} URL;

#define LIBPKI_URL_BUF_SIZE	8192

#include <libpki/pki_mem.h>
#include <libpki/net/http.h>

/* Function prototypes */
int URL_free ( URL *url );
URL *URL_new ( char *url );

PKI_MEM_STACK *URL_get_data ( char *url_s, ssize_t max_size );
PKI_MEM_STACK *URL_get_data_url ( URL *url, ssize_t max_size );
PKI_MEM_STACK *URL_get_data_file( URL *url, ssize_t size );

int URL_put_data_file ( URL *url, PKI_MEM *data );
int URL_put_data ( char *url_s, PKI_MEM *data, char *contType );
int URL_put_data_url ( URL *url, PKI_MEM *data, char *contType );

#define getParsedUrl( a ) URL_new ( a )

#endif
