/* OpenCA libpki package
* (c) 2000-2006 by Massimiliano Pala and OpenCA Group
* All Rights Reserved
*
* ===================================================================
* Released under OpenCA LICENSE
*/

#ifndef _LIBPKI_HEADER_DATA_ST_H
#define _LIBPKI_HEADER_DATA_ST_H

#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/rsa.h>
#include <openssl/dsa.h>
#include <openssl/pkcs12.h>
#include <openssl/safestack.h>

#ifdef ENABLE_ECDSA
#include <openssl/ec.h>
#endif

typedef struct EVP_PKEY 	PKI_KEYPAIR;

#define  PKI_ID			int
#define  PKI_ID_UNKNOWN		NID_undef

#define  PKI_DIGEST_ALG		EVP_MD
#define	 PKI_ALGOR 		X509_ALGOR

// typedef struct X509_NAME 	PKI_X509_NAME;
#define PKI_X509_NAME		X509_NAME

#define PKI_DIGEST_ALG_NULL  	(PKI_DIGEST_ALG *) NULL
#define PKI_DIGEST_ALG_UNKNOWN 	(PKI_DIGEST_ALG *) NULL

#define PKI_ALGOR_MD2		NID_md2
#define PKI_DIGEST_ALG_MD2	(PKI_DIGEST_ALG *) EVP_md2()
#define PKI_ALGOR_MD4		NID_md4
#define PKI_DIGEST_ALG_MD4	(PKI_DIGEST_ALG *) EVP_md4()
#define PKI_ALGOR_MD5		NID_md5
#define PKI_DIGEST_ALG_MD5	(PKI_DIGEST_ALG *) EVP_md5()
#define PKI_ALGOR_SHA1		NID_sha1
#define PKI_DIGEST_ALG_SHA1	(PKI_DIGEST_ALG *) EVP_sha1()

#ifdef NID_sha224
#define ENABLE_SHA224
#define PKI_ALGOR_SHA224	NID_sha224
#define PKI_DIGEST_ALG_SHA224	(PKI_DIGEST_ALG *) EVP_sha224()
#else
#define PKI_ALGOR_SHA224	NID_undef
#define PKI_DIGEST_ALG_SHA224	(PKI_DIGEST_ALG *) NULL
#endif

#ifdef NID_sha256
#define PKI_DIGEST_ALG_SHA256	(PKI_DIGEST_ALG *) EVP_sha256()
#define ENABLE_SHA256
#define PKI_ALGOR_SHA256	NID_sha256
#else
#define PKI_ALGOR_SHA256	NID_undef
#define PKI_DIGEST_ALG_SHA256	(PKI_DIGEST_ALG *) NULL
#endif

#ifdef NID_sha384
#define PKI_DIGEST_ALG_SHA384	(PKI_DIGEST_ALG *) EVP_sha384()
#define ENABLE_SHA384
#define PKI_ALGOR_SHA384	NID_sha384
#else
#define PKI_ALGOR_SHA384	NID_undef
#define PKI_DIGEST_ALG_SHA384	(PKI_DIGEST_ALG *) NULL
#endif

#ifdef NID_sha512
#define ENABLE_SHA512
#define PKI_ALGOR_SHA512	NID_sha512
#define PKI_DIGEST_ALG_SHA512	(PKI_DIGEST_ALG *) EVP_sha512()
#else
#define PKI_ALGOR_SHA512	NID_undef
#define PKI_DIGEST_ALG_SHA512	(PKI_DIGEST_ALG *) NULL
#endif

#ifdef NID_ripemd160
#define PKI_DIGEST_ALG_RIPEMD160	(PKI_DIGEST_ALG *) EVP_ripemd160()
#define ENABLE_RIPEMD160
#define PKI_ALGOR_RIPEMD160	NID_ripemd160
#else
#define PKI_ALGOR_RIPEMD160	NID_undef
#define PKI_DIGEST_ALG_RIPEMD160	(PKI_DIGEST_ALG *) NULL
#endif


#ifdef ENABLE_ECDSA
#define PKI_DIGEST_ALG_ECDSA_SHA1	(PKI_DIGEST_ALG *) EVP_ecdsa()
#define PKI_DIGEST_ALG_ECDSA_DSS1	(PKI_DIGEST_ALG *) EVP_ecdsa()
#else
#define PKI_DIGEST_ALG_ECDSA_DSS1	(PKI_DIGEST_ALG *) NULL
#define PKI_DIGEST_ALG_ECDSA_SHA1	(PKI_DIGEST_ALG *) NULL
#endif

#define PKI_ALGOR_NULL		(PKI_ALGOR *) NULL
#define PKI_ALGOR_UNKNOWN	NID_undef
#define PKI_ALGOR_RSA_MD2	NID_md2WithRSAEncryption
#define PKI_ALGOR_RSA_MD5	NID_md5WithRSAEncryption
#define PKI_ALGOR_RSA_MD4	NID_md4WithRSAEncryption
#define PKI_ALGOR_RSA_SHA1	NID_sha1WithRSAEncryption

#ifdef NID_sha224WithRSAEncryption
#define ENABLE_RSA_SHA_2
#endif

#ifdef ENABLE_SHA224
#define PKI_ALGOR_RSA_SHA224	NID_sha224WithRSAEncryption
#else
#define PKI_ALGOR_RSA_SHA224	NID_undef
#endif

#ifdef ENABLE_SHA256
#define PKI_DIGEST_ALG_RSA_DEFAULT		PKI_DIGEST_ALG_SHA256
#define PKI_ALGOR_RSA_SHA256	NID_sha256WithRSAEncryption
#else
#define PKI_DIGEST_ALG_RSA_DEFAULT		PKI_DIGEST_ALG_SHA1
#define PKI_ALGOR_RSA_SHA256	NID_undef
#endif

#ifdef ENABLE_SHA384
#define PKI_ALGOR_RSA_SHA384	NID_sha384WithRSAEncryption
#else
#define PKI_ALGOR_RSA_SHA384	NID_undef
#endif

#ifdef ENABLE_SHA512
#define PKI_ALGOR_RSA_SHA512	NID_sha512WithRSAEncryption
#else
#define PKI_ALGOR_RSA_SHA512	NID_undef
#endif

#ifdef ENABLE_RIPEMD160
#define PKI_ALGOR_RSA_RIPEMD160	NID_ripemd160WithRSA
#else
#define PKI_ALGOR_RSA_RIPEMD160	NID_undef
#endif

#define PKI_ALGOR_DSA_SHA1	NID_dsaWithSHA1
#define PKI_ALGOR_DSS1		60000
#define PKI_ALGOR_ECDSA_DSS1	60001
#define PKI_DIGEST_ALG_DSS1	(PKI_DIGEST_ALG *) EVP_dss1()

/* Begin - NID_dsa_with_SHA224 */
#ifdef NID_dsa_with_SHA224 
#define ENABLE_DSA_SHA_2
#define PKI_ALGOR_DSA_SHA224	NID_dsa_with_SHA224
#else
#define PKI_ALGOR_DSA_SHA224	NID_undef
#endif 
/* End - NID_dsa_with_SHA224 */

/* Begin - NID_dsa_with_SHA256 */
#ifdef NID_dsa_with_SHA256 
#define ENABLE_DSA_SHA_2
#define PKI_DIGEST_ALG_DSA_DEFAULT		PKI_DIGEST_ALG_SHA256
#define PKI_ALGOR_DSA_SHA256	NID_dsa_with_SHA256
#else
#define PKI_DIGEST_ALG_DSA_DEFAULT		PKI_DIGEST_ALG_SHA1
#define PKI_ALGOR_DSA_SHA256	NID_undef
#endif 
/* End - NID_dsa_with_SHA256 */

/* Begin - NID_ecdsa_with_SHA1 */
#ifdef NID_ecdsa_with_SHA1 
#define PKI_ALGOR_ECDSA_SHA1	NID_ecdsa_with_SHA1
#else
#define PKI_ALGOR_ECDSA_SHA1	NID_undef
#endif 
/* End - NID_ecdsa_with_SHA1 */

/* Begin - NID_ecdsa_with_224 */
#ifdef NID_ecdsa_with_SHA224
#define ENABLE_ECDSA_SHA_2
#define PKI_ALGOR_ECDSA_SHA224	NID_ecdsa_with_SHA224
#else
#define PKI_ALGOR_ECDSA_SHA224	NID_undef
#endif
/* End - NID_ecdsa_with_SHA224 */

/* Begin - NID_ecdsa_with_SHA256 */
#ifdef NID_ecdsa_with_SHA256 
#define PKI_DIGEST_ALG_ECDSA_DEFAULT		PKI_DIGEST_ALG_SHA256
#define PKI_ALGOR_ECDSA_SHA256	NID_ecdsa_with_SHA256
#else
#define PKI_DIGEST_ALG_ECDSA_DEFAULT		PKI_DIGEST_ALG_DSS1
#define PKI_ALGOR_ECDSA_SHA256	NID_undef
#endif
/* End - NID_ecdsa_with_SHA256 */

/* Begin - NID_ecdsa_with_384 */
#ifdef NID_ecdsa_with_SHA384
#define PKI_ALGOR_ECDSA_SHA384	NID_ecdsa_with_SHA384
#else
#define PKI_ALGOR_ECDSA_SHA384	NID_undef
#endif
/* End - NID_ecdsa_with_SHA384 */

/* Begin - NID_ecdsa_with_512 */
#ifdef NID_ecdsa_with_SHA512
#define PKI_ALGOR_ECDSA_SHA512	NID_ecdsa_with_SHA512
#else
#define PKI_ALGOR_ECDSA_SHA512	NID_undef
#endif
/* End - NID_ecdsa_with_SHA512 */


/* Default DIGEST algorithm */
#ifdef ENABLE_SHA256
#define PKI_DIGEST_ALG_DEFAULT		PKI_DIGEST_ALG_SHA256
#define PKI_DIGEST_ALG_ID_DEFAULT	PKI_ALGOR_SHA256
#define PKI_ALGOR_DEFAULT		PKI_ALGOR_RSA_SHA256
#else
#define PKI_DIGEST_ALG_DEFAULT		PKI_DIGEST_ALG_SHA1
#define PKI_DIGEST_ALG_ID_DEFAULT	PKI_ALGOR_SHA1
#define PKI_ALGOR_DEFAULT		PKI_ALGOR_RSA_SHA1
#endif

#define PKI_OID				ASN1_OBJECT
#define PKI_X509_EXTENSION		X509_EXTENSION

#define PKI_ALGOR_ID			int
#define PKI_X509_CERT 			X509 	
#define PKI_X509_REQ			X509_REQ 
#define PKI_X509_CRL			X509_CRL 
#define PKI_TIME			ASN1_GENERALIZEDTIME
#define PKI_INTEGER			ASN1_INTEGER

#define PKI_PKCS7			PKCS7

#define PKI_X509_CRL_ENTRY	X509_REVOKED
/* typedef struct X509_REVOKED 	PKI_CRL_ENTRY; */

typedef struct pki_digest_data {
	PKI_DIGEST_ALG *algor;
	unsigned char *digest;
	int size;
} PKI_DIGEST;

typedef struct pki_store_st {
        void	*store_ptr;
} PKI_STORE;

#include <libpki/hsm_st.h>
#include <libpki/token_st.h>

/* End of _LIBPKI_HEADER_DATA_ST_H */
#endif
