/* OpenCA libpki package
* (c) 2000-2007 by Massimiliano Pala and OpenCA Group
* All Rights Reserved
*
* ===================================================================
* Released under OpenCA LICENSE
*/

/* BEGIN_C_DECLS should be used at the beginning of your declarations,
so that C++ compilers don't mangle their names.  Use END_C_DECLS at
the end of C declarations. */
#undef BEGIN_C_DECLS
#undef END_C_DECLS
#ifdef __cplusplus
	# define BEGIN_C_DECLS extern "C" {
	# define END_C_DECLS }
#else
	# define BEGIN_C_DECLS /* empty */
	# define END_C_DECLS /* empty */
#endif
     
/* PARAMS is a macro used to wrap function prototypes, so that
compilers that don't understand ANSI C prototypes still work,
and ANSI C compilers can issue warnings about type mismatches. */
#undef PARAMS
#if defined (__STDC__) || defined (_AIX) \
	|| (defined (__mips) && defined (_SYSTYPE_SVR4)) \
		|| defined(WIN32) || defined(__cplusplus)
	# define PARAMS(protos) protos
#else
	# define PARAMS(protos) ()
#endif

#ifndef _LIBPKI_PKI_H
#define _LIBPKI_PKI_H	1

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>
#include <libxml/nanohttp.h>

//#ifdef __LIB_BUILD__
#include <libpki/config.h>
//#endif

/*
#ifdef STDC_HEADERS
# include <string.h>
# include <stdio.h>
# include <stdlib.h>
#else
# ifndef HAVE_STRCHR
#  define strchr index
#  define strrchr rindex
# endif
char *strchr (), *strrchr ();
# ifndef HAVE_MEMCPY
#  define memcpy(d, s, n) bcopy ((s), (d), (n))
#  define memmove(d, s, n) bcopy ((s), (d), (n))
# endif
#endif
*/

#include <limits.h>
#include <syslog.h>
#include <ctype.h>
#include <sys/types.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <sys/file.h>
#include <sys/times.h>

#include <sys/sem.h>
#include <sys/ipc.h>

#ifdef _ARCH_SOLARIS
#include <fcntl.h>
#endif

#if defined (__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
	/* Ok, Semafores are correctly supported */
#elif defined (_SYS_SEM_H_)
	/* OpenBSD is not a GNU_LIBRARY but knows semaphores */
#else
	/* We should define here the structure */
	union semun {
             int val;                  /* value for SETVAL */
             struct semid_ds *buf;     /* buffer for IPC_STAT, IPC_SET */
             unsigned short *array;    /* array for GETALL, SETALL */
                                       /* Linux specific part: */
             struct seminfo *__buf;    /* buffer for IPC_INFO */
       };

#endif

BEGIN_C_DECLS

#define PKI_NAMESPACE_PREFIX		"pki"
#define PKI_NAMESPACE_HREF		"http://www.openca.org/openca/pki/1/0/0"

#define PKI_SUBSCRIBER_REQ_TYPE		"application/pki-subscriber-request"
#define PKI_SUBSCRIBER_RESP_TYPE	"application/pki-subscriber-response"
#define PKI_MANAGEMENT_REQ_TYPE		"application/pki-management-request"
#define PKI_MANAGEMENT_RESP_TYPE	"application/pki-management-response"

#ifdef HAVE_ENGINE
#define ENV_OPENCA_ENGINE		"engine"
#define ENV_OPENCA_ENGINE_ID		"engine_id"
#define ENV_OPENCA_ENGINE_PRE		"engine_pre"
#define ENV_OPENCA_ENGINE_POST		"engine_post"
#endif

#define PKI_FORMAT_UNDEF    0
#define PKI_FORMAT_ASN1     1
#define PKI_FORMAT_TEXT     2
#define PKI_FORMAT_PEM      3
#define PKI_FORMAT_NETSCAPE 4
#define PKI_FORMAT_PKCS12   5
#define PKI_FORMAT_SMIME    6
#define PKI_FORMAT_ENGINE   7

/* Supported Signing schemes identifiers */
typedef enum {
	PKI_SCHEME_UNKNOWN 	= 0,
	PKI_SCHEME_RSA,
	PKI_SCHEME_DSA,
	PKI_SCHEME_DH,
	PKI_SCHEME_ECDSA
} PKI_SCHEME_ID;

#define CRL_OK			1
#define	CRL_NOT_YET_VALID	2
#define	CRL_EXPIRED		3
#define	CRL_ERROR_NEXT_UPDATE	4
#define	CRL_ERROR_LAST_UPDATE	5
#define CRL_ERROR_UNKNOWN	10

#define CRL_REASON_UNSPECIFIED			0
#define CRL_REASON_KEY_COMPROMISE		1
#define CRL_REASON_CA_COMPROMISE		2
#define CRL_REASON_AFFILIATION_CHANGED		3
#define CRL_REASON_SUPERSEDED			4
#define CRL_REASON_CESSATION_OF_OPERATION	5
#define CRL_REASON_CERTIFICATE_HOLD		6
#define CRL_REASON_REMOVE_FROM_CRL		8
#define CRL_REASON_HOLD_INSTRUCTION		9
#define CRL_REASON_KEY_TIME			10
#define CRL_REASON_CA_KEY_TIME			11

#define APP_PASS_LEN    			1024

#define PKI_ERR					0
#define PKI_OK					1

#define PKI_CONFIG  				xmlDoc
#define PKI_CONFIG_ELEMENT  			xmlNode

/* Misc Define */
#define PKI_X509_CERT_BEGIN_ARMOUR		"-----BEGIN CERTIFICATE-----"
#define PKI_X509_CERT_END_ARMOUR		"-----END CERTIFICATE-----"

/* Pthread lib include */
#include <pthread.h>

/* Libpki Includes */
#include <libpki/support.h>
#include <libpki/pki_mem.h>
#include <libpki/net/http.h>
#include <libpki/stack.h>
#include <libpki/net/url.h>
#include <libpki/net/ldap.h>
#include <libpki/net/sock.h>
#include <libpki/pki_x509_profile.h>
#include <libpki/pki_cred.h>
#include <libpki/pki_string.h>
#include <libpki/crypto.h>
#include <libpki/pki_init.h>
#include <libpki/pki_algor.h>
#include <libpki/pki_id.h>
#include <libpki/pki_oid.h>
#include <libpki/pki_digest.h>
#include <libpki/pki_conf.h>
#include <libpki/pki_keypair.h>
#include <libpki/pki_x509_name.h>
#include <libpki/pki_x509_cert.h>
#include <libpki/pki_x509_cert_file.h>
#include <libpki/pki_x509_req.h>
#include <libpki/pki_x509_req_file.h>
#include <libpki/pki_x509_crl.h>
#include <libpki/pki_x509_p12.h>
#include <libpki/pki_x509_pkcs7.h>
#include <libpki/pki_io.h>
#include <libpki/pki_time.h>
#include <libpki/pki_integer.h>
#include <libpki/drivers/hsm_main.h>
#include <libpki/drivers/engine/engine_hsm.h>
#include <libpki/drivers/engine/engine_hsm_pkey.h>
#include <libpki/profile.h>
#include <libpki/extensions.h>
#include <libpki/pki_x509_extension.h>
#include <libpki/token.h>

/* Log Subsystem Support */
#include <libpki/pki_log.h>

/* DBMS support */
#ifdef __LIB_BUILD__
#include <libpki/net/pki_mysql.h>
#include <libpki/net/pki_pg.h>
#include <libpki/net/pkcs11.h>
#endif /* END of __LIB_BUILD__ */

/* SCEP Interface */
#include <libpki/scep/scep_asn1.h>
#include <libpki/scep/scep_bio.h>
#include <libpki/scep/scep.h>
#include <libpki/scep/scep_msg.h>
#include <libpki/scep/scep_pk7.h>
#include <libpki/scep/scep_sigattr.h>

/* CMS Interface */
#include <libpki/cms.h>

/* PRQP Support */
#include <libpki/prqp/prqp.h>

END_C_DECLS

#endif
