#ifndef _LIBPKI_LOG_H
#define _LIBPKI_LOG_H

typedef enum {
	PKI_LOG_TYPE_STDERR = 0,
	PKI_LOG_TYPE_SYSLOG,
	PKI_LOG_TYPE_FILE,
	PKI_LOG_TYPE_FILE_XML,
	PKI_LOG_TYPE_DB
} PKI_LOG_TYPE;

typedef enum {
	PKI_LOG_NONE = 0,
	PKI_LOG_ERR,
	PKI_LOG_WARNING,
	PKI_LOG_NOTICE,
	PKI_LOG_INFO,
	PKI_LOG_DEBUG
} PKI_LOG_LEVEL;

typedef enum {
	PKI_LOG_FLAGS_NONE = 0,
	PKI_LOG_FLAGS_ENABLE_DEBUG   = 0x01,
	PKI_LOG_FLAGS_ENABLE_SIGNATURE = 0x02
} PKI_LOG_FLAGS;


typedef struct PKIlog_st {
	/* Keep track if the LOG subsystem has undergone initialization */
	int initialized;

	/* Type of PKI_LOG - PKI_LOG_TYPE */
	PKI_LOG_TYPE type;

	/* Identifier of the resource */
	char *resource;

	/* Log Level - one of PKI_LOG_LEVEL */
	PKI_LOG_LEVEL level;

	/* Flags for log activities - DEBUG, SIGNATURE, etc... */
	PKI_LOG_FLAGS flags;

	/* Enable Signed Log */
	PKI_TOKEN *tk;

	/* Callbacks function - init */
	int (*init)(struct PKIlog_st *);

	/* Callbacks function - add */
	void (*add)(int, char *, va_list);

	/* Callbacks function - finalize */
	int (*finalize)(struct PKIlog_st *);

	/* Callback function - sign */
	int (*entry_sign)(struct PKIlog_st *, char * );

} PKI_LOG;

int PKI_log_init ( PKI_LOG_TYPE type, PKI_LOG_LEVEL level, char *resource,
				PKI_LOG_FLAGS flags, PKI_TOKEN *tk );

void PKI_log( int level, char *fmt, ... );

void PKI_log_debug( char *fmt, ... );

int PKI_log_end( void );

#endif
