/* PKI_STRING management for libpki */

#ifndef _LIBPKI_STRING_H
#define _LIBPKI_STRING_H

#include <openssl/asn1.h>
#include <openssl/objects.h>

/* Basic Definitions */
#define PKI_STRING 			ASN1_STRING
#define PKI_BIT_STRING 			ASN1_BIT_STRING

typedef enum {
	/* Unknown */
	PKI_STRING_UNKNOWN = -1,
	/* ASCII Strings */
	PKI_STRING_IA5 = B_ASN1_IA5STRING,
	/* UTF8 Strings */
	PKI_STRING_UTF8 = B_ASN1_UTF8STRING,
	/* BIT Strings */
	PKI_STRING_BIT = B_ASN1_BIT_STRING,
	/* Two Bytes Chears */
	PKI_STRING_BMP = B_ASN1_BMPSTRING,
	/* T61 Strings - Don't use in PKIX standards */
	PKI_STRING_T61 = B_ASN1_T61STRING,
} PKI_STRING_DATATYPE;

/* Manipulating functions */
PKI_STRING * PKI_STRING_new_null ( int type );
PKI_STRING * PKI_STRING_new( int type, char * val, size_t size );
int PKI_STRING_set( PKI_STRING *s, char *content, size_t size );
int PKI_STRING_get_type( PKI_STRING *s );
char * PKI_STRING_get_utf8( PKI_STRING *s );
void PKI_STRING_free( PKI_STRING *s );

/* Printing to fd or stdout */
int PKI_STRING_print( PKI_STRING *s );
int PKI_STRING_print_fp( FILE *fp, PKI_STRING *s );

#endif


