/* PKI Resource Query Protocol Message implementation
 * (c) 2006 by Massimiliano Pala and OpenCA Group
 * All Rights Reserved
 *
 * This software is released under the GPL2 License included
 * in the archive. You can not remove this copyright notice.
 */
                                                                                
#ifndef _LIBPKI_PRQP_LIB_H
#define _LIBPKI_PRQP_LIB_H

char *i2s_ASN1_IA5STRING(X509V3_EXT_METHOD *method, ASN1_IA5STRING *ia5);
ASN1_IA5STRING *s2i_ASN1_IA5STRING(X509V3_EXT_METHOD *method,
             X509V3_CTX *ctx, char *str);
int CERT_IDENTIFIER_cmp ( CERT_IDENTIFIER *a, CERT_IDENTIFIER *b);

/* Service Objects conversion functions */
int PRQP_init_all_services ( void );

// int PRQP_s2id( char *ss );
// const char * PRQP_id2s ( int d );
// int PRQP_obj2id ( ASN1_OBJECT *a );

const char *PRQP_oid2s ( PKI_OID *a );
PKI_OID *PRQP_s2oid ( char *a );
int PRQP_srv_cmp( PKI_OID *a, PKI_OID *b );

/* Certificate Identifier */
CERT_IDENTIFIER * PRQP_CERTID_new ( PKI_X509_CERT *caCert, 
		PKI_X509_CERT *issuerCert, PKI_X509_CERT *issuedCert,
		char *subject_s, char *serial_s, PKI_DIGEST_ALG * dgst );

CERT_IDENTIFIER * PRQP_get_issuer_caId( EVP_MD *dgst, X509 *x );

CERT_IDENTIFIER *PRQP_cert_id_new( PKI_X509_NAME *caName, 
		PKI_X509_NAME *caIssuerName, ASN1_INTEGER *serial, 
		ASN1_OCTET_STRING *subCertHash, ASN1_OCTET_STRING *subKeyHash,
		ASN1_OCTET_STRING *subKeyId, ASN1_OCTET_STRING *issKeyId,
		PKI_DIGEST_ALG *dgst );
/*
CERT_IDENTIFIER *PRQP_cert_id_new( X509_NAME *caName, ASN1_INTEGER *serial, 
		ASN1_OCTET_STRING *subCertHash, ASN1_OCTET_STRING *subKeyHash,
		ASN1_OCTET_STRING *subKeyId, ASN1_OCTET_STRING *issKeyId,
		EVP_MD *dgst );
*/

/* General Signature Function for req or resp */
int PRQP_sign ( PRQP_REQ *req, PRQP_RESP *resp, PKI_KEYPAIR *pkey, 
		PKI_X509_CERT *x, PKI_DIGEST_ALG *dgst, 
		PKI_X509_CERT_STACK * certs, HSM *hsm );

// ***************** REQUEST *******************

PRQP_REQ *PRQP_REQ_new_cert(PKI_X509_CERT *x, PKI_X509_CERT *issuer, 
		PKI_X509_CERT *issued, char *issuer_s, char *serial_s, 
			PKI_DIGEST_ALG *md );

PRQP_REQ *PRQP_REQ_new_url( char * cert_s, char *issuer_cert_s, 
	char *issued_cert_s, char *issuer_s, char *serial_s, EVP_MD *md ); 

PRQP_REQ *PRQP_REQ_new_file( char * file, int format);

PRQP_REQ * PRQP_REQ_new_certs_res( PKI_X509_CERT *caCert, 
		PKI_X509_CERT *caIssuerCert, PKI_X509_CERT *issuedCert,
			PKI_STACK *sk_services );

ASN1_OCTET_STRING *PRQP_NONCE_new (int size);

/* Helper functions to add services to requests */
int PRQP_REQ_add_service_stack ( PRQP_REQ *p, PKI_STACK *sk_services );
int PRQP_REQ_add_service ( PRQP_REQ *p, char *ss );

/* TOKEN easy signature function */
int PRQP_REQ_sign_tk( PRQP_REQ *req, PKI_TOKEN *tk, PKI_DIGEST_ALG *dgst );

/* Signatures && Verification low-level functions */
int PRQP_REQ_sign( PRQP_REQ *req, PKI_KEYPAIR *pkey, PKI_X509_CERT *x,
		PKI_DIGEST_ALG *dgst, PKI_X509_CERT_STACK * certs, HSM *hsm );

int PRQP_REQ_DATA_sign (PRQP_REQ *req, PKI_KEYPAIR *pkey, 
					PKI_DIGEST_ALG *md, HSM *hsm);

int PRQP_REQ_is_signed( PRQP_REQ *r );
int PRQP_REQ_verify ( PRQP_REQ *r );
// int PRQP_REQ_DATA_verify ( PRQP_REQ *req, PKI_KEYPAIR *pkey);

void * PRQP_REQ_get_data ( PRQP_REQ *r, PRQP_DATATYPE type );

// ***************** RESPONSE ******************

int PRQP_RESP_version_set ( PRQP_RESP *resp, int ver );
TBS_RESP_DATA *PRQP_RESP_respdata_get ( PRQP_RESP *resp );
int PRQP_RESP_nonce_dup ( PRQP_RESP *resp, PRQP_REQ *req );
int PRQP_RESP_pkistatus_set ( PRQP_RESP *resp, long v, char *info );
int PRQP_RESP_new_req ( PRQP_RESP **resp_pnt, PRQP_REQ *req, int status, 
						long secs );
PKI_STACK * PRQP_RESP_url_sk ( PRQP_RESP *r );
int PRQP_RESP_sign ( PRQP_RESP *resp, PKI_KEYPAIR *pkey, PKI_X509_CERT *x,
		PKI_DIGEST_ALG *dgst, PKI_X509_CERT_STACK * certs, HSM *hsm );
/*
int PRQP_RESP_verify_signature ( PRQP_RESP *resp, PKI_X509_CERT *x,
		PKI_DIGEST_ALG *dgst, PKI_X509_CERT_STACK *certs );
*/


/* TOKEN easy signature function */
int PRQP_RESP_sign_tk( PRQP_RESP *resp, PKI_TOKEN *tk, PKI_DIGEST_ALG *dgst);

/* Signatures && Verification low-level functions */
int PRQP_RESP_sign ( PRQP_RESP *resp, PKI_KEYPAIR *pkey, PKI_X509_CERT *x,
		PKI_DIGEST_ALG *dgst, PKI_X509_CERT_STACK * certs, HSM *hsm );

int PRQP_RESP_DATA_sign (PRQP_RESP *resp, PKI_KEYPAIR *pkey, 
					PKI_DIGEST_ALG *md, HSM *hsm );

int PRQP_RESP_is_signed( PRQP_RESP *r );
int PRQP_RESP_verify ( PRQP_RESP *r );

// int PRQP_RESP_DATA_verify ( PRQP_RESP *resp, PKI_KEYPAIR *pkey);

void * PRQP_RESP_get_data ( PRQP_RESP *r, PRQP_DATATYPE type );
PKI_OID *PRQP_RESOURCE_RESPONSE_TOKEN_get_oid ( RESOURCE_RESPONSE_TOKEN *rrt );
PKI_STACK *PRQP_RESOURCE_RESPONSE_TOKEN_get_services( RESOURCE_RESPONSE_TOKEN *rrt );

// ******************** Print facilities *******************

int PRQP_REQ_print ( PRQP_REQ *req );
int PRQP_REQ_print_fp ( FILE *fp, PRQP_REQ *req);

int PRQP_RESP_print ( PRQP_RESP *resp );
int PRQP_RESP_print_fp ( FILE *fp, PRQP_RESP *resp );

#endif
