/* token.h */

#ifndef _LIBPKI_TOKEN_HEADERS_H
#define _LIBPKI_TOKEN_HEADERS_H

#include <libpki/token_st.h>

/* Memory related functions */
PKI_TOKEN *PKI_TOKEN_new_null( void );
PKI_TOKEN *PKI_TOKEN_new( char *confDir, char *name );
int PKI_TOKEN_free( PKI_TOKEN *tk );

/* Token Initialization */
int PKI_TOKEN_set_config_dir ( PKI_TOKEN *tk, char *dir );
char * PKI_TOKEN_get_config_dir ( PKI_TOKEN *tk );
int PKI_TOKEN_init (PKI_TOKEN *tk, char *conf_url, char *name);
PKI_OID *PKI_TOKEN_OID_new ( PKI_TOKEN *tk, char *oid_s );

/* Configuration options */
int PKI_TOKEN_set_scheme( PKI_TOKEN *tk, int scheme );
int PKI_TOKEN_set_algor( PKI_TOKEN *tk, PKI_ALGOR_ID algor );
int PKI_TOKEN_X509_REQ_profile_set( PKI_TOKEN *tk, PKI_X509_PROFILE *req_prof );
int PKI_TOKEN_X509_CERT_profile_set( PKI_TOKEN *tk, PKI_X509_PROFILE *cert_prof );

/* Basic functions: add data to the TOKEN structure */
int PKI_TOKEN_set_cert ( PKI_TOKEN *tk, PKI_X509_CERT *x );
int PKI_TOKEN_set_cacert ( PKI_TOKEN *tk, PKI_X509_CERT *x );
int PKI_TOKEN_set_keypair ( PKI_TOKEN *tk, PKI_KEYPAIR *pkey );
int PKI_TOKEN_set_otherCerts ( PKI_TOKEN *tk, PKI_X509_CERT_STACK *stack );
int PKI_TOKEN_set_trustedCerts ( PKI_TOKEN *tk, PKI_X509_CERT_STACK *stack );
int PKI_TOKEN_set_cred ( PKI_TOKEN *tk, PKI_CRED *cred );

/* Load data from URL pointers */
int PKI_TOKEN_load_cert( PKI_TOKEN *tk, char *url_string );
int PKI_TOKEN_load_cacert( PKI_TOKEN *tk, char *url_string );
int PKI_TOKEN_load_keypair( PKI_TOKEN *tk, char *url_string );
int PKI_TOKEN_load_otherCerts( PKI_TOKEN *tk, char *url_string );
int PKI_TOKEN_load_trustedCerts( PKI_TOKEN *tk, char *url_string );

/* Retrieve pointers to PKI_TOKEN data */
PKI_KEYPAIR *PKI_TOKEN_get_keypair ( PKI_TOKEN *tk );
PKI_X509_CERT *PKI_TOKEN_get_cert ( PKI_TOKEN *tk );
PKI_X509_CERT *PKI_TOKEN_get_cacert ( PKI_TOKEN *tk );
PKI_X509_CERT_STACK * PKI_TOKEN_get_otherCerts ( PKI_TOKEN *tk);
PKI_X509_CERT_STACK * PKI_TOKEN_get_trustedCerts ( PKI_TOKEN *tk);
PKI_CRED *PKI_TOKEN_get_cred ( PKI_TOKEN *tk );

/* Save data to URL pointers */
int PKI_TOKEN_write ( PKI_TOKEN *tk, char *url_string, int format);
int PKI_TOKEN_write_cert ( PKI_TOKEN *tk, char *url_string, int format );
int PKI_TOKEN_write_req ( PKI_TOKEN *tk, char *url_string, int format );
int PKI_TOKEN_write_keypair ( PKI_TOKEN *tk, char *url_string, int format );
int PKI_TOKEN_write_otherCerts ( PKI_TOKEN *tk, char *url_string, int format );
int PKI_TOKEN_write_trustedCerts (PKI_TOKEN *tk, char *url_string, int format);

/* TOKEN operations */
int PKI_TOKEN_new_keypair ( PKI_TOKEN *tk, int bits, char *label );
int PKI_TOKEN_new_req(PKI_TOKEN *tk, char *subject, char *profile_s );
int PKI_TOKEN_self_sign (PKI_TOKEN *tk, char *subject, char *serial,
							char *profile_s );
PKI_X509_CERT* PKI_TOKEN_issue_cert(PKI_TOKEN *tk, char *subject, char *serial,
					 PKI_X509_REQ *req, char *profile_s);

/* TOKEN profile ops */
int PKI_TOKEN_load_profiles ( PKI_TOKEN *tk, char *urlStr );
PKI_X509_PROFILE *PKI_TOKEN_search_profile( PKI_TOKEN *tk, char *profile_s );

#endif

